#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.django.db`` -- Database Utilities
"""

from django.conf import settings


class RattailRouter(object):
    """
    Database router which will ensure all Rattail models are "bound" to the
    Rattail database instead of the default Django database.

    .. .. note::
    ..    This router also prevents the ``syncdb`` command from creating tables
    ..    for Rattail models.  This is done in order to keep database migration
    ..    logic in the Rattail domain (i.e. using ``alembic``).
    """

    def __init__(self):
        self.db_alias = getattr(settings, 'RATTAIL_DATABASE_ALIAS', 'rattail')
        self.managed = bool(getattr(settings, 'RATTAIL_DJANGO_MANAGED', False))

    def db_for_read(self, model, **hints):
        if model._meta.app_label == 'rattail':
            return self.db_alias

    def db_for_write(self, model, **hints):
        if model._meta.app_label == 'rattail':
            return self.db_alias

    def allow_syncdb(self, db, model):
        if model._meta.app_label == 'rattail':
            return self.managed
