#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.django.rattail.models.auth`` -- Models for 'auth' Extension
"""

from __future__ import absolute_import

from django.db import models

from rattail.django.rattail.models import Model, uuid_field


__all__ = ['Role', 'User', 'UserRole']


class Role(Model):
    """
    Represents a role within the system; used to manage permissions.
    """

    class Meta(Model.Meta):
        db_table = Model.prefix('roles')

    uuid = uuid_field()
    name = models.CharField(max_length=25, unique=True)

    def __repr__(self):
        return "<Role: %s>" % self.name

    def __unicode__(self):
        return unicode(self.name or '')


class User(Model):
    """
    Represents a user of the system.  This may or may not correspond to a real
    person, i.e. some users may exist solely for automated tasks.
    """

    class Meta(Model.Meta):
        db_table = Model.prefix('users')

    uuid = uuid_field()
    username = models.CharField(max_length=25, unique=True)
    password = models.CharField(max_length=60, blank=True, null=True)
    salt = models.CharField(max_length=29, blank=True, null=True)
    person = models.ForeignKey('Person', db_column='person_uuid', blank=True, null=True)

    def __repr__(self):
        return "<User: %s>" % self.username

    def __unicode__(self):
        return unicode(self.username or '')

    @property
    def display_name(self):
        """
        Returns :attr:`rattail.Person.display_name` if present; otherwise
        returns :attr:`username`.
        """

        if self.person and self.person.display_name:
            return self.person.display_name
        return self.username


class UserRole(Model):
    """
    Represents the association between a :class:`User` and a :class:`Role`.
    """

    class Meta(Model.Meta):
        db_table = Model.prefix('users_roles')

    uuid = uuid_field()
    user = models.ForeignKey(User, db_column='user_uuid')
    role = models.ForeignKey(Role, db_column='role_uuid')

    def __repr__(self):
        return "<UserRole: %s, %s>" % (self.user, self.role)

    def __unicode__(self):
        return unicode(self.role or '')
