#!/usr/bin/env python

from rattail.django.rattail.tests.models import ModelTestCase
from rattail.django.rattail import models


class ModelTests(ModelTestCase):

    def test_managed(self):
        model = self.model()
        self.assertTrue(model._meta.managed)
        self.assertTrue(model._meta.db_table.startswith('rattail_'))


class PhoneNumberTests(ModelTests):
    model = models.PhoneNumber

    def test_repr(self):
        phone = models.PhoneNumber()
        phone.number = '800-555-1234'
        self.assertEqual(repr(phone), '<PhoneNumber: 800-555-1234>')

    def test_unicode(self):
        phone = models.PhoneNumber()
        phone.number = '800-555-1234'
        self.assertEqual(unicode(phone), u'800-555-1234')


class EmailAddressTests(ModelTests):
    model = models.EmailAddress

    def test_repr(self):
        email = models.EmailAddress()
        email.address = 'test@edbob.org'
        self.assertEqual(repr(email), '<EmailAddress: test@edbob.org>')

    def test_unicode(self):
        email = models.EmailAddress()
        email.address = 'test@edbob.org'
        self.assertEqual(unicode(email), u'test@edbob.org')


class PostalAddressTests(ModelTests):
    model = models.PostalAddress

    def test_repr(self):
        address = models.PostalAddress()
        address.street = '123 Main St.'
        self.assertEqual(repr(address), '<PostalAddress: 123 Main St.>')

    def test_unicode(self):
        address = models.PostalAddress()
        address.street = '123 Main St.'
        self.assertEqual(unicode(address), u'123 Main St.')


class ContactTestsMixin(object):

    def test_get_info_dict(self):
        model = self.model()
        self.assertRaises(ValueError, model._get_info_dict, object)

    def test_get_info_key(self):
        model = self.model()
        self.assertRaises(ValueError, model._get_info_key, object)

    def test_update_phone(self):
        model = self.model()
        phone = model.update_phone('test', dict(
                number='800-555-1234'))
        self.assertTrue(phone)
        self.assertEqual(phone.number, '800-555-1234')

    def test_get_phone(self):
        model = self.model()
        phone = model.update_phone('test', dict(
                number='800-555-1234'))
        self.assertTrue(phone)
        self.assertIs(model.get_phone('test'), phone)

    def test_set_preferred_phone_single(self):
        model = self.model()
        phone = model.update_phone('test', dict(
                number='800-555-1234'))
        self.assertTrue(phone)
        model.set_preferred_phone(phone)
        self.assertIs(model.get_preferred_phone(), phone)

    def test_set_preferred_phone_multiple(self):
        model = self.model()
        phone1 = model.update_phone('test1', dict(
                number='800-555-1234'))
        self.assertTrue(phone1)
        phone2 = model.update_phone('test2', dict(
                number='800-555-6789'))
        self.assertTrue(phone2)
        model.set_preferred_phone(phone2)
        self.assertIs(model.get_preferred_phone(), phone2)
        self.assertEqual(phone2.preference, 1)
        phone1 = model.get_phone('test1')
        self.assertEqual(phone1.preference, 2)

    def test_get_preferred_phone(self):
        model = self.model()
        phone = model.update_phone('test', dict(
                number='800-555-1234'))
        self.assertTrue(phone)
        model.set_preferred_phone(phone)
        self.assertIs(model.get_preferred_phone(), phone)
        self.assertIs(model.phone, phone)

        model._preferred.clear()
        phone = model.get_preferred_phone()
        self.assertEqual(phone.number, '800-555-1234')

        phone.delete()
        model._preferred.clear()
        self.assertIs(model.phone, None)

    def test_update_email(self):
        model = self.model()
        email = model.update_email('test', dict(
                address='test@edbob.org'))
        self.assertTrue(email)
        self.assertEqual(email.address, 'test@edbob.org')

    def test_get_email(self):
        model = self.model()
        email = model.update_email('test', dict(
                address='test@edbob.org'))
        self.assertTrue(email)
        self.assertIs(model.get_email('test'), email)

    def test_set_preferred_email_single(self):
        model = self.model()
        email = model.update_email('test', dict(
                address='test@edbob.org'))
        self.assertTrue(email)
        model.set_preferred_email(email)
        self.assertIs(model.get_preferred_email(), email)

    def test_set_preferred_email_multiple(self):
        model = self.model()
        email1 = model.update_email('email1', dict(
                address='test@edbob.org'))
        self.assertTrue(email1)
        email2 = model.update_email('test2', dict(
                address='test@edbob.org'))
        self.assertTrue(email2)
        model.set_preferred_email(email2)
        self.assertIs(model.get_preferred_email(), email2)
        self.assertEqual(email2.preference, 1)
        phone1 = model.get_email('test1')
        self.assertEqual(email1.preference, 2)

    def test_get_preferred_email(self):
        model = self.model()
        email = model.update_email('test', dict(
                address='test@edbob.org'))
        self.assertTrue(email)
        model.set_preferred_email(email)
        self.assertIs(model.get_preferred_email(), email)
        self.assertIs(model.email, email)

        model._preferred.clear()
        email = model.get_preferred_email()
        self.assertEqual(email.address, 'test@edbob.org')

        email.delete()
        model._preferred.clear()
        self.assertIs(model.email, None)

    def test_update_address(self):
        model = self.model()
        address = model.update_address('test', dict(
                street='123 Main St.'))
        self.assertTrue(address)
        self.assertEqual(address.street, '123 Main St.')

    def test_get_address(self):
        model = self.model()
        address = model.update_address('test', dict(
                street='123 Main St.'))
        self.assertTrue(address)
        self.assertIs(model.get_address('test'), address)

    def test_set_preferred_address_single(self):
        model = self.model()
        address = model.update_address('test', dict(
                street='123 Main St.'))
        self.assertTrue(address)
        model.set_preferred_address(address)
        self.assertIs(model.get_preferred_address(), address)

    def test_set_preferred_address_multiple(self):
        model = self.model()
        address1 = model.update_address('address1', dict(
                street='123 Main St.'))
        self.assertTrue(address1)
        address2 = model.update_address('test2', dict(
                street='123 Main St.'))
        self.assertTrue(address2)
        model.set_preferred_address(address2)
        self.assertIs(model.get_preferred_address(), address2)
        self.assertEqual(address2.preference, 1)
        phone1 = model.get_address('test1')
        self.assertEqual(address1.preference, 2)

    def test_get_preferred_address(self):
        model = self.model()
        address = model.update_address('test', dict(
                street='123 Main St.'))
        self.assertTrue(address)
        model.set_preferred_address(address)
        self.assertIs(model.get_preferred_address(), address)
        self.assertIs(model.address, address)

        model._preferred.clear()
        address = model.get_preferred_address()
        self.assertEqual(address.street, '123 Main St.')

        address.delete()
        model._preferred.clear()
        self.assertIs(model.address, None)


class PersonTests(ModelTests, ContactTestsMixin):
    model = models.Person

    def test_repr(self):
        person = models.Person()
        person.display_name = 'Fred Flintstone'
        self.assertEqual(repr(person), '<Person: Fred Flintstone>')

    def test_unicode(self):
        person = models.Person()
        person.display_name = 'Fred Flintstone'
        self.assertEqual(unicode(person), u'Fred Flintstone')

    def test_display_name(self):
        person = models.Person()
        person.first_name = 'Fred'
        person.last_name = 'Flintstone'
        person.update_display_name()
        self.assertEqual(person.display_name, 'Fred Flintstone')

    def test_display_name_first(self):
        person = models.Person()
        person.first_name = 'Fred'
        person.update_display_name()
        self.assertEqual(person.display_name, 'Fred')

    def test_display_name_last(self):
        person = models.Person()
        person.last_name = 'Flintstone'
        person.update_display_name()
        self.assertEqual(person.display_name, 'Flintstone')

    def test_display_name_save(self):
        person = models.Person()
        person.first_name = 'Fred'
        person.last_name = 'Flintstone'
        person.save()
        self.assertEqual(person.display_name, 'Fred Flintstone')

    def test_employee(self):
        person = models.Person()
        self.assertIs(person.employee, None)
        person.save()
        employee = models.Employee()
        employee.person = person
        employee.save()
        self.assertTrue(person.employee)

    def test_make_employee(self):
        person = models.Person()
        self.assertIs(person.employee, None)
        person.make_employee()
        self.assertTrue(person.employee)
        # Once more for full code coverage...
        person.make_employee()

    def test_unmake_employee(self):
        person = models.Person()
        person.make_employee()
        self.assertTrue(person.employee)
        person.unmake_employee()
        self.assertIs(person.employee, None)


class PartyTests(ModelTests):
    model = models.Party

    def test_unicode(self):
        person = models.Person()
        person.display_name = 'Fred Flintstone'
        party = models.Party()
        party.person = person
        party.type = 'test'
        self.assertEqual(unicode(party), u'Fred Flintstone (test)')


class PartyManagerTestsMixin(object):
    models = models.Party

    def test_queryset(self):
        # TODO: This should really assert something...
        qs = self.model.objects
        qs.all()


class RoleTests(ModelTests):
    model = models.Role

    def test_repr(self):
        role = models.Role()
        role.name = 'Test'
        self.assertEqual(repr(role), '<Role: Test>')

    def test_unicode(self):
        role = models.Role()
        role.name = 'Test'
        self.assertEqual(unicode(role), u'Test')


class UserTests(ModelTests):
    model = models.User

    def test_repr(self):
        user = models.User()
        user.username = 'test'
        self.assertEqual(repr(user), '<User: test>')

    def test_unicode(self):
        user = models.User()
        user.username = 'test'
        self.assertEqual(unicode(user), u'test')

    def test_display_name(self):
        user = models.User()
        user.username = 'test'
        self.assertEqual(user.display_name, 'test')

    def test_person_display_name(self):
        person = models.Person()
        person.display_name = 'Fred Flintstone'
        user = models.User()
        user.person = person
        self.assertEqual(user.display_name, 'Fred Flintstone')


class UserRoleTests(ModelTests):
    model = models.UserRole

    def test_repr(self):
        user = models.User()
        user.username = 'test'
        role = models.Role()
        role.name = 'Test'
        user_role = models.UserRole()
        user_role.user = user
        user_role.role = role
        self.assertEqual(repr(user_role), '<UserRole: test, Test>')

    def test_unicode(self):
        role = models.Role()
        role.name = 'Test'
        user_role = models.UserRole()
        user_role.role = role
        self.assertEqual(unicode(user_role), u'Test')


class StoreTests(ModelTests):
    model = models.Store


class DepartmentTests(ModelTests):
    model = models.Department


class SubdepartmentTests(ModelTests):
    model = models.Subdepartment


class CategoryTests(ModelTests):
    model = models.Category


class BrandTests(ModelTests):
    model = models.Brand


class VendorTests(ModelTests):
    model = models.Vendor


class ProductTests(ModelTests):
    model = models.Product


class ProductPriceTests(ModelTests):
    model = models.ProductPrice


class ProductCostTests(ModelTests):
    model = models.ProductCost


class EmployeeTests(ModelTests):
    model = models.Employee


class CustomerGroupTests(ModelTests):
    model = models.CustomerGroup


class CustomerTests(ModelTests):
    model = models.Customer


class CustomerGroupAssignmentTests(ModelTests):
    model = models.CustomerGroupAssignment


class CustomerPartyTests(PartyTests, PartyManagerTestsMixin):
    model = models.CustomerParty


class LabelProfileTests(ModelTests):
    model = models.LabelProfile
