#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.django.rattail.views`` -- Views
"""

from django.http import HttpResponse
from django.utils import simplejson as json

from pyzipcode import ZipCodeDatabase


zcdb = ZipCodeDatabase()


def get_zipcode_data(request):
    """
    Returns everything knowable about a given zipcode, as a JSON string.
    Returns an empty string if the zipcode is not found in the database.
    """

    zipcode = request.GET.get('zipcode')
    zipcode = zcdb.get(zipcode)
    if not zipcode:
        return HttpResponse('')

    zipcode = zipcode[0]
    data = {
        'city':         zipcode.city,
        'state':        zipcode.state,
        'zip':          zipcode.zip,
        'latitude':     zipcode.latitude,
        'longitude':    zipcode.longitude,
        'timezone':     zipcode.timezone,
        'dst':          zipcode.dst,
        }
    return HttpResponse(json.dumps(data), content_type='application/json')
